#################################################################################
#  Script test.tcl: Creates multiple stations when some send data, some send 
#		    voice and other send video. Each station will have a priority 
#		    for sending its packets which will be changed by changing 
#		    values of DIFS_, CWMin_ and etc... 
#  Written by: 	    Albert Vainer, Benny Yoshpa, Alex Shraer
#  Date:	    03/2004
#################################################################################
if {$argc < 2} {
	puts stderr "usage: ns $argv0 <number of voice stations> <E is 1\\0>"
	exit 1;
}

# ======================================================================
# Define options
# ======================================================================

set val(chan)       	Channel/WirelessChannel
set val(prop)       	Propagation/TwoRayGround
set val(netif)      	Phy/WirelessPhy
set val(mac)        	Mac/802_11
set val(ifq)        	Queue/DropTail/PriQueue
set val(ll)         	LL
set val(ant)       	Antenna/OmniAntenna
set val(x)              670   ;# X dimension of the topography
set val(y)              670   ;# Y dimension of the topography
set val(ifqlen)         2000            ;# max packet in ifq
set val(seed)           0.0
set val(adhocRouting)   DSDV
set val(stop)           30.0           ;# simulation time

set epsilon 0.00005

set in_voice [lindex $argv 0]
puts "$in_voice";
set voice [expr $in_voice + 1]
set back_ground 4					;# number of phone streams (Voice)
set ppktsize 160				;# pktsize of phone streams
set pinterval 0.020        ;#equivalent to send rate of 8000 bytes 
set pktsize 1500				;# pktsize for background traffic
set num_nodes [expr $voice + $back_ground]       ;# number of mobilenodes in the scenario
set transport both

set sink 0
set vsrc 1

Phy/WirelessPhy set per_	0.0
#Phy/WirelessPhy set CPThresh_ .000040
#Phy/WirelessPhy set CSThresh_ .000010
#Phy/WirelessPhy set RXThresh_ .000030

Mac/802_11 set bandwidth_	11Mb    

#MAC_MIB set RTSThreshold_	3000
#MAC_MIB set ShortRetryLimit_	7
#MAC_MIB set LongRetryLimit_	4


Mac/802_11 set SlotTime_      	0.000010 
Mac/802_11 set SIFS_	      	0.000020
Mac/802_11 set CWMax_         	1023	 ;# it's going to be the same for all station

#PHY_MIB set CWMin_0		15
#PHY_MIB set CWOffset_0		0


# =====================================================================
# Main Program
# ======================================================================
#
# Initialize Global Variables
#
# create simulator instance
set ns_		[new Simulator]
# setup topography object
set topo	[new Topography]
# create trace object for ns and nam

set tracefd	[open test-out_$in_voice.tr w]
set namtrace    [open test-out_$in_voice.nam w]

$ns_ trace-all $tracefd
$ns_ namtrace-all-wireless $namtrace $val(x) $val(y)

# define topology
$topo load_flatgrid $val(x) $val(y)

#
# Create God
#
set god_ [create-god $num_nodes]


# num_bss_nodes describes the number of nodes in the BSS; the
# remaining nodes are ADHOC; default: all nodes are in the BSS
if {![info exists num_bss_nodes]} {
	set num_bss_nodes $num_nodes
}

set chan_1_ [new $val(chan)] 

set e_enb [lindex $argv 1]


Mac/802_11 set E_enable $e_enb

###### Defining specific stations #########
######           VOICE	 	  #########

#Mac/802_11 set DIFS_          0.000080 ;#  <===========    Decide later   Benny
#Mac/802_11 set CWMin_         63
#Mac/802_11 set PF_	       2
Mac/802_11 set DIFS_          0.000020 ;#  <===========    Decide later   Benny
Mac/802_11 set CWMin_         15
Mac/802_11 set PF_	      2


$ns_ node-config -adhocRouting $val(adhocRouting) \
                 -llType $val(ll) \
                 -macType $val(mac) \
                 -ifqType $val(ifq) \
                 -ifqLen $val(ifqlen) \
                 -antType $val(ant) \
                 -propType $val(prop) \
                 -phyType $val(netif) \
		 -channel $chan_1_ \
                 -topoInstance $topo \
		 -agentTrace ON \
                 -routerTrace OFF \
                 -macTrace OFF

#
#  Create the specified number of nodes $voice and "attach" them
#  to the channel. 

for {set i 0} {$i < $voice} {incr i} {
	set node_($i) [$ns_ node]	
	$node_($i) random-motion 0		;# disable random motion
}

#################### Voice flows definitions #################################
for {set i 1} {$i < $voice} {incr i} {
		set P		[new Application/Traffic/CBR]
		set P_src	[new Agent/UDP]
		set P_sink	[new Agent/UDP]

		$P set		random_ 0
		$P set		packetSize_ $ppktsize
		$P set		interval_ $pinterval
		$P_src set	packetSize_ $ppktsize
		$P_src set	class_ [expr $i + 1] 
	#	$P_src set	prio_ $pri

		if {[expr $i % 2] == 0} {
			set node_src	$i
			set node_sink	0
		} else {
			set node_src	0
			set node_sink	$i
		}
		$ns_ attach-agent $node_($node_src) $P_src
		$ns_ attach-agent $node_($node_sink) $P_sink
		$ns_ connect $P_src $P_sink
		$P attach-agent $P_src
puts "P$i with pktsize $ppktsize flows from Node$node_src to Node$node_sink every $pinterval"

	#	$ns_ at [expr 3.0 * $i] "$P start"
	#	$ns_ at [expr 3.0 * ($i + $num_nodes) + $soaktime - 6] "$P stop"
		$ns_ at 0.0 "$P start"
		$ns_ at $val(stop) "$P stop"

	}

###### Defining specific stations #########
######           DATA	 	  #########
Mac/802_11 set DIFS_          0.000080 ;#  <===========    Decide later   Benny
Mac/802_11 set CWMin_         63
Mac/802_11 set PF_	      2

$ns_ node-config -adhocRouting $val(adhocRouting) \
                 -llType $val(ll) \
                 -macType $val(mac) \
                 -ifqType $val(ifq) \
                 -ifqLen $val(ifqlen) \
                 -antType $val(ant) \
                 -propType $val(prop) \
                 -phyType $val(netif) \
		 -channel $chan_1_ \
                 -topoInstance $topo \
		 -agentTrace ON \
                 -routerTrace OFF \
                 -macTrace OFF 

#
#  Create the specified number of nodes $voice and "attach" them
#  to the channel. 

for {set i $voice} {$i < $num_nodes} {incr i} {
	set node_($i) [$ns_ node]	
	$node_($i) random-motion 0		;# disable random motion
}


##########################################################
# Setup Background flows

	for {set i $voice} {$i < $num_nodes} {incr i} {

		set agent_src	Agent/TCP
		set agent_sink	Agent/TCPSink/DelAck
		set B		[new Application/FTP]
		set B_src	[new $agent_src]
		set B_sink	[new $agent_sink]

		$B_src		set packetSize_ $pktsize
		$B_src		set class_ [expr $i + 1]
	
		if { $i%2 == 0} {
			set sink $i
			set src 0
		} else {
			set src $i
			set sink 0
		}
		

		$ns_ attach-agent $node_($src) $B_src
		$ns_ attach-agent $node_($sink) $B_sink
		$ns_ connect $B_src $B_sink
		$B attach-agent $B_src
	puts "$agent_src B$i with pktsize $pktsize flows from Node$src to Node$sink"

		$ns_ at 0.0 "$B start"
		$ns_ at $val(stop) "$B stop"
	}


###########################################################################################
# 
# Define node movement model
#
#puts "Loading connection pattern..."
#source $val(cp)

# 
# Define traffic model
#
#puts "Loading scenario file..."
#source $val(sc)

# Define node initial position in nam

for {set i 0} {$i < $num_nodes} {incr i} {

    # 20 defines the node size in nam, must adjust it according to your scenario
    # The function must be called after mobility model is defined
    
    $ns_ initial_node_pos $node_($i) 20
}


# End of simulation conditions/operations

for {set i 0} {$i < $num_nodes } {incr i} {
    $ns_ at $val(stop) "$node_($i) reset";
#    $node_($i) set X_ [expr $i*30.0]
#    $node_($i) set Y_ [expr $i*30.0]
#    $node_($i) set Z_ 0.0

}
#for {set i 0} {$i < $voice } {incr i} {
#    $ns_ at $val(stop) "$node_($i) reset";
#}

$ns_ at $val(stop) "do_stop"


set delta 0.01
set endtime [expr $val(stop) + $delta]
$ns_ at $val(stop) "do_halt"

proc do_stop {} {
	global ns_ tracefd namtrace
	$ns_ flush-trace
	close $tracefd
	close $namtrace
#	exec ~/nam alex.nam &
}

proc do_halt {} {
	global ns_
	puts "NS EXITING..."
	$ns_ halt
}


###########################################################################################
# Start up the simulation

puts "Starting Simulation..."
$ns_ run


